#pragma once
#include <PSX/GPU/gpu_primitives.hpp>
#include <PSX/GPU/gpu.hpp>
#include <PSX/GTE/gte.hpp>

namespace GTETest {
    using namespace JabyEngine;

    struct GTE_Sprite {
        GPU::AreaI16     area;
        GPU::PositionI16 pivot;
        deg_t            angle;
        GPU::POLY_FT4    display;

        static constexpr GTE_Sprite create(const GPU::POLY_FT4& base) {
            return GTE_Sprite{
                .area    = GPU::AreaI16::create(base.get_rect_pos(), base.get_rect_size()),
                .pivot   = GPU::PositionI16::create(base.get_rect_size().width/2, base.get_rect_size().height/2),
                .angle   = 0.0_deg,
                .display = base
            };
        }

        void apply(const GTE::MATRIX& gbl_matrix) {
            auto matrix = GTE::MATRIX::translated(-this->pivot.x, -this->pivot.y, 0).comp(
                GTE::MATRIX::rotated(0.0_deg, 0.0_deg, this->angle)
            ).comp(
                GTE::MATRIX::translated(this->pivot.x, this->pivot.y, 0)
            ).comp(
                GTE::MATRIX::translated(this->area.position.x, this->area.position.y)
            ).comp(gbl_matrix);

            this->display.vertex0 = matrix.apply_to(GPU::Vertex::create(0,                                          0)); 
            this->display.vertex1 = matrix.apply_to(GPU::Vertex::create(this->area.size.width,                      0)); 
            this->display.vertex2 = matrix.apply_to(GPU::Vertex::create(0,                     this->area.size.height)); 
            this->display.vertex3 = matrix.apply_to(GPU::Vertex::create(this->area.size.width, this->area.size.height)); 
        }

        void render() {
            GPU::render(this->display);
        }
    };
}